#! /usr/bin/perl -w

use POE qw(Session);


#################
# Server Session
#

sub server_spawn {
    my $alias = shift;
    POE::Session->create
        ( 'inline_states' => 
	  { '_start' => \&server_start,
	    'print' => \&server_print,
	    'shutdown' => \&server_shutdown,
	  },
          'args' => [ $alias ],
        );
    undef;
}

# Handler fr _start-Event
sub server_start {
    my ($kernel, $heap, $alias) = @_[KERNEL, HEAP, ARG0];

    print "Server start\n";
    $heap->{'alias'} = $alias;
    # als Argument nur der Name des Alias
    $kernel->alias_set($alias);
}

# Handler fr print-Event
sub server_print {
    my ($string, $postback) = @_[ARG0, ARG1];

    print "Server print: $string\n";
    # ggf. Rckgabewert via postback, 
    # Aufruf der Code-Ref mit eigenen Argumenten
    &{$postback}("printed") if defined $postback;
    return "printed";
}

# Handler fr shutdown-Event
sub server_shutdown {
    my ($kernel, $heap) = @_[KERNEL, HEAP];

    $kernel->alias_remove($heap->{alias});
    print "Server shutdown\n";
}


#################
# client session
#

sub client_spawn {
    my $server = shift;

    POE::Session->create
        ( 'inline_states' => 
	  { '_start' => \&client_start,
	    'result' => \&client_result,
	  },
          'heap' => { 'server' => $server },
        );
    undef;
}

# Handler fr _start-Event
sub client_start {
    my ($kernel, $session, $heap) = @_[KERNEL, SESSION, HEAP];
    my $server = $heap->{'server'};

    # Parameter fuer post/call Aufrufe:
    #  Ziel-Session, Event-Name, dem Event zugeordnete Argumente
    $kernel->post($server, 'print',
                  'spaeter, da asynchron, ohne Rckgabewert');
    my $result = $kernel->call($server, 'print',
                               'zuerst, da synchron, mit Rckgabewert');
    print "client/call: $result\n";
    # Argumente fuer postback: Event-Namen den die (hier aktuelle) Session
    # erhalten soll, dem Event zugeordnete Argumente
    # zurueckgegeben wird eine Code-Referenz
    $kernel->post($server, 'print', 'zuletzt, mit spterem postback',
                 $session->postback('result', '2. post'));
    $kernel->post($server, 'shutdown');
}

# Handler fr result
sub client_result {
    my ($eigene_args, $uebergebene_args) = @_[ARG0, ARG1];

    print "client/postback:\n eigene argumente: ".
      join(', ', @$eigene_args) . 
	"\n vom Aufrufenden uebergebene argumente: ".
	  join(', ', @$uebergebene_args) . "\n";
}

##############
#
# Sessions erstellen
server_spawn('server');
client_spawn('server');
#
# Start der POE Event-Loop der Kern-Instanz
$poe_kernel->run();

