#! /usr/bin/perl -w

use POE qw(Session Filter::Stream Wheel::ReadWrite Wheel::SocketFactory
           Driver::SysRW);

### host:port auf dem gehoert werden soll
my $bind_host = 'localhost';
my $bind_port = 12345;
### host:port auf den weitergeleitet werden soll
my $dest_host = 'localhost';
my $dest_port = 21;


##################
# listen/accept Session
#
sub la_spawn {
    POE::Session->create
        ( 'inline_states' => 
	  { '_start'   => \&la_start,
	    'connection' => \&la_connection,
	    'failure'    => \&la_failure,
	  },
        );
    undef;
}


sub la_start {
    my $heap = $_[HEAP];
    # neue Socket-Factory erstellen
    $heap->{'socketfactory'} = POE::Wheel::SocketFactory->new
      ( 'BindAddress'   => $bind_host,
	'BindPort'      => $bind_port,
	'SuccessState'  => 'connection',
	'FailureState'  => 'failure',
	Reuse => 'yes',
      );
    warn 'created '.$heap->{'socketfactory'}->ID();
}


sub la_connection {
    my ($socket, $remote_addr, 
	$remote_port, $wheel_id) = @_[ARG0..ARG3];

    warn "l/a: neue verbindung $wheel_id mit $remote_addr:$remote_port\n";
    fw_spawn($socket);
}


sub la_failure {
    my ($heap, $operation, $errnum, 
	$errstr, $wheel_id) = @_[HEAP, ARG0..ARG3];
    warn "l/a: socketfactory $wheel_id, $operation: $errnum/$errstr\n";
    delete $heap->{'socketfactory'};
}


##################
# Weiterleitende/forwarding Session
#
sub fw_spawn {
    my $socket = shift;
    POE::Session->create
        ( 'inline_states' => 
	  { '_start'     => \&fw_start,
	    'connected'  => \&fw_connected,
	    'input_dest' => \&fw_input_dest,
	   ' input_src'  => \&fw_input_src,
	    'failure'    => \&fw_failure,
	  },
          'heap' => { 'accepted_socket' => $socket },
        );
    undef;
}


sub fw_start {
    my $heap = $_[HEAP];
    $heap->{'socketfactory'} = POE::Wheel::SocketFactory->new
      ( 'RemoteAddress' => $dest_host,
        'RemotePort'    => $dest_port,
        'SuccessState'  => 'connected',
        'FailureState'  => 'failure',
      );
    warn 'fw: neue socketfactory '.$heap->{'socketfactory'}->ID()."\n";
}


sub fw_connected {
    my ($heap, $socket) = @_[HEAP, ARG0];
    # Wheels fuer Ziel und Quelle
    $heap->{'wheel_dest'} = POE::Wheel::ReadWrite->new
      ( 'Handle'     => $socket,
        'Filter'     => POE::Filter::Stream->new(),
        'Driver'     => POE::Driver::SysRW->new(),
        'InputState' => 'input_dest',
        'ErrorState' => 'failure',
      );
    $heap->{'wheel_src'} = POE::Wheel::ReadWrite->new
      ( 'Handle'     => delete $heap->{accepted_socket},
        'Filter'     => POE::Filter::Stream->new(),
        'Driver'     => POE::Driver::SysRW->new(),
        'InputState' => 'input_src',
        'ErrorState' => 'failure',
      );
    warn 'fw: neue Wheels fr Ziel/Quelle: '.
      $heap->{'wheel_dest'}->ID().'/'.$heap->{'wheel_src'}->ID();
}


sub fw_input_dest {
    $_[HEAP]->{'wheel_src'}->put($_[ARG0]);
}


sub fw_input_src {
    $_[HEAP]->{'wheel_dest'}->put($_[ARG0]);
}


sub fw_failure {
    my ($heap, $operation, $errnum, 
	$errstr, $wheel_id) = @_[HEAP, ARG0..ARG3];
    warn "fw: Wheel $wheel_id, $operation: $errnum/$errstr\n";
    foreach (qw(socketfactory wheel_dest wheel_src accepted_socket)) {
        delete $heap->{$_};
    }
}



la_spawn();
$poe_kernel->run();
